from typing import Any, Dict
import requests


def webhook_callback(rules: Dict[str, Any]) -> callable:
    def wrapper(*args, **kwargs):
        response = requests.post(
            rules.get("route"),
            json=kwargs.get("data"),
            headers=kwargs.get("headers"))

        if rules.get("callback"):
            rules.get("callback")(response)

    return wrapper



from threading import Thread
from typing import Any, Dict, Union
from onany.manager import EventManager
from onany.web import webhook_callback


def clear_listeners():
    EventManager.clear()


def dispatch(event: str, *args, **kwargs) -> None:
    return EventManager.emit(event, *args, **kwargs)


def disthread(event: str, *args, **kwargs) -> Thread:
    thread: Thread = Thread(
        target=EventManager.emit,
        args=(event, *args,),
        kwargs=kwargs)
    thread.start()

    return thread


def listener(event: str, callback: Union[callable, Dict[str, Any]] = None) -> callable:
    if callable(callback):
        return EventManager.listen(event, callback)

    if callback and isinstance(callback, dict):
        return EventManager.listen(
            event, 
            webhook_callback(callback))

    def wrapper(callback: callable) -> None:
        return EventManager.listen(event, callback)

    return wrapper



from typing import Dict, List


class EventManager:
    _events_pool: Dict[str, List[callable]] = {}

    @classmethod
    def clear(cls):
        cls._events_pool = {}

    @classmethod
    def listen(cls, event: str, callback: callable):
        cls._register_event(event)
        cls._events_pool[event].append(callback)

    @classmethod
    def emit(cls, event: str, *args, **kwargs):
        cls._register_event(event)

        for callback in cls._events_pool[event]:
            callback(*args, **kwargs)

    @classmethod
    def _register_event(cls, event: str):
        if event not in cls._events_pool:
            cls._events_pool[event] = []



import os
from setuptools import setup


with open(os.path.join("README.rst")) as file:
    readme = file.read()


with open(os.path.join("requirements.txt")) as file:
    requirements = file.read().split("\n")

setup(
    name="onany",
    packages=["onany"],
    version=os.getenv("GITHUB_REF") \
        .replace("refs/tags/v", ""),
    license="MIT",
    description="Event manager library",
    long_description=readme,
    author="Vinicius Guedes",
    author_email="viniciusgued@gmail.com",
    url="https://github.com/vinyguedess/onany",
    download_url="https://github.com/vinyguedess/onany/archive/master.zip",
    keywords=["event", "events", "listener", "dispatch"],
    
    install_requires=requirements,
    
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8'
    ])
